import java.util.*;

public class OrderedList<E> {
	// Overview: Tipo modificabile che rappresenta una lista ordinata di elementi di E

	// AF: <elemens.get(first), elems(ordine.get(first)), elems(ordine.get(ordine.get(first))), ...>
	// IR: dim = elemens.size() = ordine.size() && dim >= 0 && 
	//       forall i. 0 <= i < dim-1 ==> elemens.get(i) != null &&
	//       forall i. 0 <= i < dim-1 ==> (-1 <= ordine.get(i) <= dim-1 && ordine.get(i) != first)
 	//       forall i, j. 0 <= i, j < dim-1 && i != j ==> ordine.get(i) != ordine.get(j)
 	//       forall i. 0 <= i < dim-1 && ordine.get(i) = k ==> (k = -1 || elemens(i) <= elemens(k))
 	//       forall i. 0 <= i < dim-1 ==> || elemens(first) <= elemens(k)

	private ArrayList<E> elems;
	private ArrayList<Integer> ordine;
    	private int dim, first;

     	/* restituisce l'indice del predecessore nella lista (non vuota) rispetto all'ordine dell'elemento elem,
	    e -1 se elem e' il nuovo minimo */
	private int pred(E elem) {		
		if (elems.get(elem).compareTo(elems.get(first))<=0) return -1;

		int res = first, next = ordine.get(res);
 		while (elems.get(elem).compareTo(elems.get(res))>0) {
			if ((next == -1) || (elems.get(elem).compareTo(elems.get(next))<=0)) return res;
			res = next;
			next = ordine.get(res);
		}

		return res;	
	}
     /* 
      REQUIRES: la lista elems non e' vuota e elem != null <== come mai qua "!= null" non inficia la strategia difensiva?
      EFFECTS: cerca il precedessore di elem nella lista rispetto all'ordinamento e se lo trova ne restituisce l'indice,
      altrimenti restituisce -1
      */  

	/* inserisce l'elemento elem nella lista rispettando l'ordine */
	public void insert(E elem) throws NullPointerException {
     		if (elem == null) throw new NullPointerException();

		if (elems.isEmpty()) {
			first = 0;
			elems.add(elem);
			ordine.add(-1);
			dim++;
			return;
		}

     		int tmp = pred(elem);
				
		if (tmp == -1) first = dim;
		
		elems.add(elem);
		ordine.add(ordine.get(tmp));
		ordine.set(tmp, dim);

	         dim++;
	}
	// Era accettabile una soluzione nel quale pred non era fornito, ma solo invocato

     /* 
      MODIFIES: this
      EFFECTS: se elem != null lo inserisce nella lista rispettando l'ordinamento,
      altrimenti lancia una NullPointerException (eccezione disponibile in Java, unchecked)
      */  
      
     /* restituisce ed elimina dalla lista l'elemento minimo rispetto all'ordine */
     public E extractMin() throws EmptyListException { 
    
    	if (dim == 0) throw EmptyListException();
     
	return null; // solo per farlo compilare     
      }
  
     /* 
      MODIFIES: this
      EFFECTS: se la lista non e' vuota toglie toglie dalla lista l'elemento minimo della lista e lo
      restituisce, altrimenti lancia una EmptyListException (eccezione non disponibile in Java, checked)
      */  
}	